@@ -1,3 +1,8 @@
+.prepare_calcregion <- function(calcregion, imgdim, psf, finesample)
+{
+  .Call('R_profit_adjust_mask', calcregion, imgdim, psf, finesample)
+}
+
 .profitParsePSF <- function(psf, modellist, psfdim=dim(psf), finesample=1L)
 {
   haspsf = length(psf) > 0
@@ -31,7 +36,7 @@ profitDataBenchmark <- function(modellist, calcregion, imgdim,
   finesample=1L, psf=NULL, fitpsf=FALSE, omp_threads=NULL, openclenv=NULL,
   openclenv_int=openclenv, openclenv_conv=openclenv,
   nbenchmark=0L, nbenchint=nbenchmark, nbenchconv=nbenchmark,
-  benchintmethods=c("brute"), benchconvmethods = c("brute","fft"),
+  benchintmethods=c("brute"), benchconvmethods = c("brute","fftw"),
   benchprecisions="double", benchconvprecisions=benchprecisions,
   benchintprecisions=benchprecisions,
   benchopenclenvs = profitGetOpenCLEnvs(make.envs = TRUE),
@@ -52,8 +57,7 @@ profitDataBenchmark <- function(modellist, calcregion, imgdim,
   if((length(benchintmethods) > 1) && nbenchint > 0)
   {
     # Hopefully avoids a pointless allocation of an empty image, but maybe not
-    if(finesample==1) image = calcregion
-    else image = matrix(0,imgdim[1],imgdim[2])
+    image = matrix(0,imgdim[1],imgdim[2])
     benches$benchint = profitBenchmark(image=image, modellist = modellist,
       nbench = nbenchint, methods = benchintmethods, precisions = benchintprecisions,
       openclenvs = benchopenclenvs, omp_threads = omp_threads, finesample = finesample)
@@ -150,7 +154,7 @@ profitSetupData=function(image, region, sigma, segim, mask, modellist,
   like.func="norm", magmu=FALSE, verbose=FALSE, omp_threads = NULL,
   openclenv=NULL, openclenv_int=openclenv, openclenv_conv=openclenv,
   nbenchmark=0L, nbenchint=nbenchmark, nbenchconv=nbenchmark,
-  benchintmethods=c("brute"), benchconvmethods = c("brute","fft"),
+  benchintmethods=c("brute"), benchconvmethods = c("brute","fftw"),
   benchprecisions="double", benchconvprecisions=benchprecisions,
   benchintprecisions=benchprecisions,
   benchopenclenvs = profitGetOpenCLEnvs(make.envs = TRUE),
@@ -259,8 +263,6 @@ profitSetupData=function(image, region, sigma, segim, mask, modellist,
     psf = psf/sum(psf)
   }
   
-  calcregion = profitUpsample(region,finesample)
-  
   if (!is.null(openclenv)) {
     if (class(openclenv) == "externalptr") {
       openclenv = openclenv
@@ -269,23 +271,9 @@ profitSetupData=function(image, region, sigma, segim, mask, modellist,
       openclenv = profitOpenCLEnv()
     }
   }
-  
-  if(haspsf)
-  {
-    dimcr = dim(calcregion)
-    calcxy = dimcr
-    if(is.null(benchconvmethods) || ("brute" %in% benchconvmethods))
-    {
-      newregion = matrix(0,calcxy[1],calcxy[2])
-      newregion[,] = calcregion
-      # TODO: Replace with profitConvolver
-      # Note: We use brute force convolution here because FFTs have ~1e-12 noise. It can be very slow, though
-      calcregion=profitConvolvePSF(newregion,psf+1,options=list(method="Bruteconv"))
-      calcregion=calcregion>0
-    } else {
-      calcregion = matrix(TRUE,calcxy[1],calcxy[2])
-    }
-  }
+
+  calcregion = .prepare_calcregion(region, imagedim, psf, finesample)
+
   # Note this actually stores whether we are fitting the PSF image for convolution with extended sources
   # It should probably be renamed fitpsfimg but will remain as such for backwards compatibility for now
   fitpsf = psftype == "analytical" && any(unlist(tofit$psf)) && any(!(names(modellist) %in% c("psf","pointsource","sky")))
