@@ -523,17 +523,20 @@ std::vector<std::string> split(const std::string &s, const std::string &delims)
 	return tokens;
 }
 
+static inline bool not_space(char c)
+{
+	return !std::isspace(c);
+}
+
 // trim from start
 static inline std::string &ltrim(std::string &s) {
-	s.erase(s.begin(), std::find_if(s.begin(), s.end(),
-	        std::not1(std::ptr_fun<int, int>(std::isspace))));
+	s.erase(s.begin(), std::find_if(s.begin(), s.end(), not_space));
 	return s;
 }
 
 // trim from end
 static inline std::string &rtrim(std::string &s) {
-	s.erase(std::find_if(s.rbegin(), s.rend(),
-	        std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
+	s.erase(std::find_if(s.rbegin(), s.rend(), not_space).base(), s.end());
 	return s;
 }
 
